@@ -7,7 +7,15 @@ class Test {
 	}
 
 	public static void main(String[] args) {
-		libc c = (libc)CInvoke.load("libc.so.6", libc.class);
+		String libname;
+		String osname = System.getProperty("os.name");
+		if (osname.equals("Mac OS X"))
+			libname = "libc.dylib";
+		else if (osname.equals("SunOS"))
+			libname = "libc.so.1";
+		else
+			libname = "libc.so.6";
+		libc c = (libc)CInvoke.load(libname, libc.class);
 
 		System.out.println("You entered: " + c.getpass("Enter password: "));
 
