@@ -32,6 +32,16 @@ POSSIBILITY OF SUCH DAMAGE.
 #include <string.h>
 #include <time.h>
 
+#ifdef _MSC_VER
+#define snprintf _snprintf
+#endif
+
+#ifdef _WIN32
+#define DLLEXPORT __declspec(dllexport)
+#else
+#define DLLEXPORT
+#endif
+
 // stack reading functions, we define our own so we can specify
 // the exact semantics/errors we want
 void expected(lua_State *l, const char *exp, int index) {
@@ -380,8 +390,9 @@ char getcode(lua_State *l, const char *family, int tblindex, int callback) {
 		}
 		return 'p';
 	} else if (!strcmp(family, "basic")) {
+		lua_Integer charid;
 		lua_getfield(l, tblindex, "charid");
-		lua_Integer charid = tointeger(l, -1);
+		charid = tointeger(l, -1);
 		lua_pop(l, 1);
 		return (char)charid;
 	} else {
@@ -484,26 +495,26 @@ CInvFunction *parsefunction(lua_State *l, CInvContext *ctx, int startarg,
 int get_arrelement_size(lua_State *l, int type) {
 	int ret;
 	lua_getfield(l, type, "size");
-	ret = tointeger(l, -1);
+	ret = (int)tointeger(l, -1);
 	lua_pop(l, 1);
 	return ret;
 }
 void push_ptr_val(lua_State *l, void *ptr) {
-	char buf[16];
-	sprintf(buf, "%p", ptr);
-	lua_pushstring(l, buf);
+	lua_pushlightuserdata(l, ptr);
 }
 void *get_ptr_val(lua_State *l, int index) {
 	void *ret;
 	const char *pstr;
 	if (lua_isnil(l, index))
 		return 0;
-	pstr = tostring(l, index);
+	if(lua_isuserdata(l, index))
+		return lua_touserdata(l, index);
+	pstr = lua_tostring(l, index);
 	if (strlen(pstr) == 0) return 0;
 	if (strlen(pstr) == 1 ||
 		strncmp("0x", pstr, 2)) {
 		char *endptr;
-		ret = (void *)strtol(pstr, &endptr, 10);
+		ret = (void *)strtol(pstr, &endptr, 16);
 		if (*endptr != '\0') {
 			lua_pushstring(l, "invalid pointer value");
 			lua_error(l);
@@ -644,7 +655,7 @@ void *marshal_param(lua_State *l, int typeindex, int argindex) {
 			if (lua_isnil(l, argindex))
 				*ptr = NULL;
 			else {
-				int arrlen = lua_objlen(l, argindex);
+				int arrlen = (int)lua_objlen(l, argindex);
 				int elsize = get_arrelement_size(l, typeindex);
 				*ptr = malloc(arrlen * elsize);
 				if (*ptr) {
@@ -729,13 +740,13 @@ void unmarshal_retval(lua_State *l, int typeindex, void *value) {
 			lua_pushnumber(l, *(long *)value);
 			break;
 		case CINV_T_EXTRALONG:
-			lua_pushnumber(l, *(long long *)value);
+			lua_pushnumber(l, (lua_Number)*(long long *)value);
 			break;
 		case CINV_T_FLOAT:
 			lua_pushnumber(l, *(float *)value);
 			break;
 		case CINV_T_DOUBLE:
-			lua_pushnumber(l, *(float *)value);
+			lua_pushnumber(l, *(double *)value);
 			break;
 		case CINV_T_PTR:
 			push_ptr_val(l, *(void **)value);
@@ -790,7 +801,7 @@ void unmarshal_struct(lua_State *l, int typeindex, void *instance) {
 }
 void unmarshal_array(lua_State *l, int typeindex, void *value, int outindex) {
 	char *cptr = *(char **)value;
-	int arrlen = lua_objlen(l, outindex), i;
+	int arrlen = (int)lua_objlen(l, outindex), i;
 	int elsize = get_arrelement_size(l, typeindex);
 
 	for (i = 1; i <= arrlen; i++) {
@@ -852,7 +863,7 @@ void _ccallback_invoked(CInvFunction *f, void *parameters[],
 
 	lua_getfield(cb->l, usertable, "params");
 	pindex = lua_gettop(cb->l);
-	numargs = lua_objlen(cb->l, pindex);
+	numargs = (int)lua_objlen(cb->l, pindex);
 
 	for (i = 0; i < numargs; i++) {
 		lua_pushinteger(cb->l, i + 1);
@@ -1033,7 +1044,7 @@ int _function_call(lua_State *l) {
 
 	lua_getfield(l, 1, "params");
 	pindex = lua_gettop(l);
-	numargs = lua_objlen(l, pindex);
+	numargs = (int)lua_objlen(l, pindex);
 	if (numargs > 0) {
 		parameters = malloc(numargs * sizeof(void*));
 		for (i = 0; i < numargs; i++) {
@@ -1292,7 +1303,7 @@ int _cinv_string_to_chararray(lua_State *l) {
 	for (i = 0; i < len; i++) {
 		lua_pushinteger(l, i + 1);
 		lua_pushlstring(l, s + i, 1);
-		lua_settable(l, 1);
+		lua_settable(l, -3);
 	}
 
 	return 1;
@@ -1344,13 +1355,13 @@ int _cinv_chararray_to_string(lua_State *l) {
 int _cinv_ptr_to_string(lua_State *l) {
 	char *ptr;
 	int len = -1;
-	if (lua_gettop(l) != 1 || lua_gettop(l) != 2) {
+	if (lua_gettop(l) != 1 && lua_gettop(l) != 2) {
 		lua_pushstring(l, "usage: cinv.ptr_to_string(cptr[, len])");
 		lua_error(l);
 	}
 	ptr = get_ptr_val(l, 1);
 	if (!lua_gettop(l) == 2) {
-		len = tointeger(l, 2);
+		len = (int)tointeger(l, 2);
 		if (len < 0) {
 			lua_pushstring(l, "invalid length parameter");
 			lua_error(l);
@@ -1384,7 +1395,7 @@ int _cinv_ptr_to_array(lua_State *l) {
 	if (value == NULL)
 		lua_pushnil(l);
 	else {
-		len = tointeger(l, 3);
+		len = (int)tointeger(l, 3);
 		// create table of desired len and call unmarshal_array
 		lua_newtable(l);
 		index = lua_gettop(l);
@@ -1437,7 +1448,7 @@ int _cinv_ptr_incr(lua_State *l) {
 		lua_error(l);
 	}
 
-	num = tointeger(l, 2);
+	num = (int)tointeger(l, 2);
 	ptr = get_ptr_val(l, 1);
 
 	ptr += num;
@@ -1480,8 +1491,7 @@ int _cinv_array(lua_State *l) {
 	return 1;
 }
 
-
-int luaopen_cinvoke_lua(lua_State *l) {
+DLLEXPORT int luaopen_cinvoke_lua(lua_State *l) {
 	declbasic(l, "Cchar", CINV_T_CHAR, 'c', 1);
 	declbasic(l, "Cshort", CINV_T_SHORT, 's', sizeof(short));
 	declbasic(l, "Cint", CINV_T_INT, 'i', sizeof(int));
